Imports System.Globalization
''' <summary>
''' Class to create or change Word documents
''' </summary>
Public Class DocumentCreator

#Region " Fields / Events "
    ''' <summary>
    ''' Information if Word installation was checked.
    ''' </summary>
    Private Shared m_WordWasChecked As Boolean = False
    ''' <summary>
    ''' Information if Word was found on check.
    ''' </summary>
    Private Shared m_IsWordInstalled As Boolean = True
    ''' <summary>
    ''' List of known extensions for Word documents.
    ''' </summary>
    Private Shared m_DocExtensions As New List(Of String)
    ''' <summary>
    ''' List of known extensions for Word templates.
    ''' </summary>
    Private Shared m_DotExtensions As New List(Of String)
    ''' <summary>
    ''' Is raised, when Word was not found.
    ''' </summary>
    Public Shared Event IsWordInstalled_Changed()
#End Region

#Region " Properties "
    ''' <summary>
    ''' List of known extensions for Word documents.
    ''' </summary>
    Shared ReadOnly Property WordDocumentExtensions() As List(Of String)
        Get
            If WordInstalledCheck() Then
                Return m_DocExtensions
            Else
                Return Nothing
            End If
        End Get
    End Property
    ''' <summary>
    ''' List of known extensions for Word templates.
    ''' </summary>
    Shared ReadOnly Property WordTemplateExtensions() As List(Of String)
        Get
            If WordInstalledCheck() Then
                Return m_DotExtensions
            Else
                Return Nothing
            End If
        End Get
    End Property
    ''' <summary>
    ''' Returns if Word was found.
    ''' </summary>
    Shared Property IsWordInstalled() As Boolean
        Get
            Return WordInstalledCheck()
        End Get
        Private Set(ByVal value As Boolean)
            If m_IsWordInstalled <> value Then
                m_IsWordInstalled = value
                RaiseEvent IsWordInstalled_Changed()
            End If
        End Set
    End Property
#End Region

#Region " Methods "
    ''' <summary>
    ''' Checks if Word is installed.
    ''' </summary>
    Private Shared Function WordInstalledCheck() As Boolean

        If Not m_WordWasChecked Then
            m_WordWasChecked = True

            Dim WdApp As New WordApplication
            IsWordInstalled = WdApp.WordExists
            If WdApp.VersionNumber > 11 Then
                m_DocExtensions.Add(".docx")
                m_DotExtensions.Add(".dotx")
                m_DocExtensions.Add(".doc")
                m_DotExtensions.Add(".dot")
            Else
                m_DocExtensions.Add(".doc")
                m_DotExtensions.Add(".dot")
            End If

            WdApp.Close()


            If Not m_IsWordInstalled Then
                Dim Message As String
                If Threading.Thread.CurrentThread.CurrentCulture.Name.ToLower.StartsWith("de") Then
                    Message = "Aktion nicht ausfhrbar. Word kann nicht gefunden werden."
                Else
                    Message = "Action can not be executed. Word can not be found."
                End If
                System.Windows.Forms.MessageBox.Show(Message)
            End If
        End If

        Return m_IsWordInstalled
    End Function
    ''' <summary>
    ''' Opens an existing Word document.
    ''' </summary>
    ''' <param name="Path">Path of the document file.</param>
    ''' <returns>True if success</returns>
    Shared Function OpenDocument(ByVal Path As String) As Boolean

        Dim Result As Boolean = False
        If Not WordInstalledCheck() Then Return False
        If Not IO.File.Exists(Path) Then Return False

        Using WdApp As New WordApplication
            Using Doc As WordDocument = WdApp.OpenDocument(Path)
                WdApp.Visible = True
                WdApp.Activate()
                Doc.Activate()
                Doc.Dispose()
                Result = True
            End Using
            WdApp.Dispose()
        End Using

        Return Result
    End Function
    ''' <summary>
    ''' Reads hidden text from a document.
    ''' </summary>
    ''' <param name="Path">Path of the document file.</param>
    Shared Function ReadHiddenText(ByVal Path As String) As String
        If Not WordInstalledCheck() Then Return ""

        If Not IO.File.Exists(Path) Then Throw New IO.FileNotFoundException
        Dim Result As String = ""
        Using WdApp As New WordApplication
            Using Doc As WordDocument = WdApp.OpenDocument(Path)
                If Doc IsNot Nothing Then
                    Result = Doc.ReadHiddenText
                    Doc.Close()
                End If
            End Using
            WdApp.Close()
        End Using

        If Result IsNot Nothing Then
            Return Result.Trim
        Else
            Return ""
        End If
    End Function
    ''' <summary>
    ''' Writes hidden text into a Word document.
    ''' </summary>
    ''' <param name="Path">Path to open or create a document.</param>
    ''' <param name="HiddenText">Text value.</param>
    Shared Function WriteHiddenText(ByVal Path As String, ByVal HiddenText As String) As Boolean
        If Not WordInstalledCheck() Then Return False

        Using WdApp As New WordApplication
            Dim IsTemplate As Boolean = False
            For Each txt As String In m_DotExtensions
                If Path.ToLower.EndsWith(txt) Then
                    IsTemplate = True
                    Exit For
                End If
            Next

            Using Doc As WordDocument = WdApp.OpenDocument(Path, "")
                If Doc Is Nothing Then Return False

                Doc.WriteHiddenText(HiddenText)
                Doc.SaveAs(Path)

                Threading.Thread.Sleep(250)

                Doc.Dispose()
            End Using
            WdApp.Dispose()

        End Using

        Return True
    End Function

    ''' <summary>
    ''' Creates a new Word document.
    ''' </summary>
    ''' <param name="Rng">Range descripition of the document contents.</param>
    ''' <param name="Path">Path of the new or refreshed document.</param>
    ''' <param name="TemplatePath">Path of used template (if document is new).</param>
    ''' <param name="FirstPageTopMargin">Top magin of first section page.</param>
    ''' <returns>True if success</returns>
    Shared Function CreateDocument(ByVal Rng As Document.Range, ByVal Path As String, Optional ByVal TemplatePath As String = "", Optional ByVal FirstPageTopMargin As Single = 0.0F) As Boolean
        If Not WordInstalledCheck() Then Return False

        Using WdApp As New WordApplication
            Dim IsTemplate As Boolean = False
            For Each txt As String In m_DotExtensions
                If Path.ToLower.EndsWith(txt) Then
                    IsTemplate = True
                    Exit For
                End If
            Next

            Using Doc As WordDocument = WdApp.OpenDocument(Path, TemplatePath, IsTemplate, True)
                If Doc Is Nothing Then Return False

                Doc.Visible = True
                Doc.Activate()
                Doc.Write(Rng, FirstPageTopMargin)

                Doc.SaveAs(Path)

                Threading.Thread.Sleep(250)

                Doc.Dispose()
            End Using
            WdApp.Dispose()
        End Using

        Return True
    End Function

#End Region

End Class


