Namespace Document
    ''' <summary>
    ''' Describes a bookmark in the document range.
    ''' </summary>
    Public Class Bookmark
        Inherits WritableObject

#Region " Fields "
        ''' <summary>
        ''' Bookmark name.
        ''' </summary>
        Private m_Name As String
        ''' <summary>
        ''' Content range of the bookmark.
        ''' </summary>
        Private m_Range As New Range
        ''' <summary>
        ''' True if this bookmark should be deleted after filling-in.
        ''' </summary>
        Private m_RemoveAfterFill As Boolean = False
        ''' <summary>
        ''' True if this bookmark should be selected.
        ''' </summary>
        Private m_Selected As Boolean = False
#End Region

#Region " Constructor "
        ''' <summary>
        ''' Creates a new bookmark with name.
        ''' </summary>
        ''' <param name="Name">Bookmark name.</param>
        Friend Sub New(ByVal Name As String)
            MyBase.New()
            m_Name = Name
        End Sub
#End Region

#Region " Properties "
        ''' <summary>
        ''' Name of the bookmark.
        ''' </summary>
        Friend ReadOnly Property Name() As String
            Get
                Return m_Name
            End Get
        End Property
        ''' <summary>
        ''' True if this bookmark should be selected.
        ''' </summary>
        WriteOnly Property Selected() As Boolean
            Set(ByVal value As Boolean)
                m_Selected = value
            End Set
        End Property
        ''' <summary>
        ''' Gets or sets the content range of the bookmark.
        ''' </summary>
        Property Range() As Range
            Get
                Return m_Range
            End Get
            Set(ByVal value As Range)
                m_Range = value
            End Set
        End Property
#End Region

#Region " Methods "
        ''' <summary>
        ''' Writes the content of this bookmark into a Word range.
        ''' </summary>
        ''' <param name="Rng">Active Word range.</param>
        Friend Overrides Sub Write(ByRef Rng As Object)

            Dim OldStart As Integer = Rng.Start
            m_Range.Write(Rng)
            Rng.Start = OldStart

            Dim Marks As Object = Rng.Bookmarks
            Marks.Add(m_Name, Rng)
            ReleaseObject(Marks)

            Rng.Collapse(WdCollapseDirection.wdCollapseEnd)
        End Sub
        ''' <summary>
        ''' Fills a bookmark in an existing document.
        ''' </summary>
        ''' <param name="DocumentRange">Full range of a Word document.</param>
        Friend Sub Fill(ByVal DocumentRange As Object)

            Dim Marks As Object = DocumentRange.Bookmarks

            If Marks.Exists(m_Name) Then

                Dim BM1 As Object = Marks.Item(m_Name)
                Dim Rng As Object = BM1.Range

                Dim OldStart As Integer = Rng.Start
                m_Range.Write(Rng)
                Rng.Start = OldStart

                If m_Selected Then
                    Rng.Select()
                End If

                If m_RemoveAfterFill Then
                    BM1.Delete()
                Else
                    Marks.Add(m_Name, Rng)
                End If

                ReleaseObject(Rng)

                ReleaseObject(BM1)

            End If
            ReleaseObject(Marks)
        End Sub
        ''' <summary>
        ''' Releases marshalled objects.
        ''' </summary>
        Private Sub ReleaseObject(ByVal obj As Object)
            If obj IsNot Nothing Then
                Try
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(obj)
                Catch
                Finally
                    obj = Nothing
                End Try
            End If
        End Sub
#End Region

    End Class
End Namespace