Namespace Document
    ''' <summary>
    ''' Describes a table in the document range.
    ''' </summary>
    Public Class Table
        Inherits WritableObject

#Region " Fields "
        ''' <summary>
        ''' Count of columns.
        ''' </summary>
        Private m_NumColumns As Integer = 0
        ''' <summary>
        ''' Count of rows.
        ''' </summary>
        Private m_NumRows As Integer = 0
        ''' <summary>
        ''' Array of table cells.
        ''' </summary>
        Private m_Cell(,) As Cell
        ''' <summary>
        ''' ReadOnly enumerable list of table cells.
        ''' </summary>
        Private m_Cells As New Cells
        ''' <summary>
        ''' ReadOnly enumerable list of table rows.
        ''' </summary>
        Private m_Rows As New Rows
        ''' <summary>
        ''' ReadOnly enumerable list of table columns.
        ''' </summary>
        Private m_Columns As New Columns
        ''' <summary>
        ''' Alignment of the table rows.
        ''' </summary>
        Private m_Alignment As WdRowAlignment = WdRowAlignment.NotSet
        ''' <summary>
        ''' Behavior to autofit table columns.
        ''' </summary>
        Private m_AutoFitBehavior As WdAutoFitBehavior = WdAutoFitBehavior.NotSet
#End Region

#Region " Constructor "
        ''' <summary>
        ''' Constructor for a new table.
        ''' </summary>
        ''' <param name="NumRows">Number of rows.</param>
        ''' <param name="NumColumns">Number of columns.</param>
        Friend Sub New(ByVal NumRows As Integer, ByVal NumColumns As Integer)
            If NumRows < 1 OrElse NumColumns < 1 Then
                Throw New Exception("A table must have at least one row and one column.")
            End If
            m_NumRows = NumRows
            m_NumColumns = NumColumns

            ReDim m_Cell(NumRows - 1, NumColumns - 1)
            For I As Integer = 0 To NumRows - 1
                For J As Integer = 0 To NumColumns - 1
                    m_Cell(I, J) = New Cell(I, J)
                    m_Cells.Add(m_Cell(I, J))
                Next
            Next

            For I As Integer = 0 To NumRows - 1
                m_Rows.Add(New Row(Me, I))
            Next
            For I As Integer = 0 To NumColumns - 1
                m_Columns.Add(New Column(Me, I))
            Next
        End Sub
#End Region

#Region " Properties "
        ''' <summary>
        ''' Count of columns.
        ''' </summary>
        ReadOnly Property ColumnCount() As Integer
            Get
                Return m_NumColumns
            End Get
        End Property
        ''' <summary>
        ''' Count of rows.
        ''' </summary>
        ReadOnly Property RowCount() As Integer
            Get
                Return m_NumRows
            End Get
        End Property
        ''' <summary>
        ''' Returns a cell by row and column index.
        ''' </summary>
        ''' <param name="RowIndex">Index of row.</param>
        ''' <param name="ColumnIndex">Index of column.</param>
        ReadOnly Property Cell(ByVal RowIndex As Integer, ByVal ColumnIndex As Integer) As Cell
            Get
                Return m_Cell(RowIndex, ColumnIndex)
            End Get
        End Property
        ''' <summary>
        ''' List of all colums.
        ''' </summary>
        ReadOnly Property Columns() As Columns
            Get
                Return m_Columns
            End Get
        End Property
        ''' <summary>
        ''' List of all rows.
        ''' </summary>
        ReadOnly Property Rows() As Rows
            Get
                Return m_Rows
            End Get
        End Property
        ''' <summary>
        ''' Gets or sets the alignment of the table rows.
        ''' </summary>
        Public Property Alignment() As WdRowAlignment
            Get
                Return m_Alignment
            End Get
            Set(ByVal value As WdRowAlignment)
                m_Alignment = value
            End Set
        End Property
        ''' <summary>
        ''' Gets or sets the behavior to autofit table columns.
        ''' </summary>
        Property AutoFitBehavior() As WdAutoFitBehavior
            Get
                Return m_AutoFitBehavior
            End Get
            Set(ByVal value As WdAutoFitBehavior)
                m_AutoFitBehavior = value
            End Set
        End Property
        ''' <summary>
        ''' List of table cells.
        ''' </summary>
        ReadOnly Property Cells() As Cells
            Get
                Return m_Cells
            End Get
        End Property
#End Region

#Region " Methods "
        ''' <summary>
        ''' Writes the content of this table into a Word range.
        ''' </summary>
        ''' <param name="Rng">Active Word range.</param>
        Friend Overrides Sub Write(ByRef Rng As Object)
            Dim Tables As Object = Rng.Tables
            Dim Tbl As Object = Tables.Add(Rng, m_NumRows, m_NumColumns)
            ReleaseObject(Tables)

            If m_Alignment <> WdRowAlignment.NotSet Then
                Dim Rows As Object = Tbl.Rows
                Rows.Alignment = m_Alignment
                ReleaseObject(Rows)
            End If

            Dim Borders As Object = Tbl.Borders
            Borders.InsideLineStyle = WdLineStyle.wdLineStyleNone
            Borders.OutsideLineStyle = WdLineStyle.wdLineStyleNone
            ReleaseObject(Borders)

            For K As Integer = 0 To m_NumRows - 1
                For L As Integer = 0 To m_NumColumns - 1
                    Dim Cl As Object = Tbl.Cell(K + 1, L + 1)
                    Dim RngCl As Object = Cl.Range
                    RngCl.Collapse(WdCollapseDirection.wdCollapseStart)
                    m_Cell(K, L).Range.Write(RngCl)
                    If m_Cell(K, L).VerticalAlignment <> WdCellVerticalAlignment.NotSet Then
                        Cl.VerticalAlignment = m_Cell(K, L).VerticalAlignment
                    End If

                    Dim Borders1 As Object = Cl.Borders

                    For I As Integer = -4 To -1
                        If m_Cell(K, L).Borders.Item(I) <> WdLineStyle.wdLineStyleNotSet Then
                            Dim Itm As Object = Borders1.Item(I)
                            Itm.LineStyle = m_Cell(K, L).Borders.Item(I)
                            ReleaseObject(Itm)
                        End If
                    Next

                    ReleaseObject(Borders1)
                    ReleaseObject(RngCl)
                    ReleaseObject(Cl)
                Next
            Next


            If m_AutoFitBehavior <> WdAutoFitBehavior.NotSet Then
                Tbl.AutoFitBehavior(m_AutoFitBehavior)
            End If

            Dim Cols As Object = Tbl.Columns

            For I As Integer = 0 To m_NumColumns - 1
                If m_Columns(I).PreferredWidth <> 0.0F Then
                    Dim Col As Object = Cols.Item(I + 1)
                    Col.PreferredWidthType = 3
                    Col.PreferredWidth = m_Columns(I).PreferredWidth
                    ReleaseObject(Col)
                End If
            Next

            ReleaseObject(Cols)

            For K As Integer = 0 To m_NumRows - 1
                For L As Integer = 0 To m_NumColumns - 1
                    If m_Cell(K, L).MergeTo IsNot Nothing Then
                        Dim MergeTo As Cell = m_Cell(K, L).MergeTo
                        Dim TabCl1 As Object = Tbl.Cell(K + 1, L + 1)
                        Dim TabCl2 As Object = Tbl.Cell(MergeTo.RowIndex + 1, MergeTo.ColumnIndex + 1)

                        TabCl1.Merge(TabCl2)

                        ReleaseObject(TabCl2)
                        ReleaseObject(TabCl1)
                    End If
                Next
            Next

            Rng = Tbl.Range
            ReleaseObject(Tbl)
            Rng.Collapse(WdCollapseDirection.wdCollapseEnd)

        End Sub
        ''' <summary>
        ''' Releases marshalled objects.
        ''' </summary>
        Private Sub ReleaseObject(ByVal obj As Object)
            If obj IsNot Nothing Then
                Try
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(obj)

                Catch
                Finally
                    obj = Nothing
                End Try
            End If
        End Sub
#End Region

    End Class
End Namespace