''' <summary>
''' Class to hold a Word application reference and to dispose it after use
''' </summary>
Friend Class WordApplication
    Implements IDisposable

#Region " Fields "
    ''' <summary>
    ''' Reference to the Word application.
    ''' </summary>
    Private m_WdApp As Object = Nothing
    ''' <summary>
    ''' True, if the Word application was open at start.
    ''' </summary>
    Private m_WordWasOpen As Boolean = False
    ''' <summary>
    ''' Support for IDisposible.
    ''' </summary>
    Private m_disposedValue As Boolean = False
#End Region

#Region " Constructor "
    ''' <summary>
    ''' Constructor.
    ''' </summary>
    Public Sub New()
        Try
            m_WdApp = GetObject(, "Word.Application")
            m_WordWasOpen = True
        Catch ex As Exception
            Try
                m_WdApp = CreateObject("Word.Application")
            Catch ex1 As Exception

            End Try
        End Try
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' True, if the Word application was open at start.
    ''' </summary>
    ReadOnly Property WasOpenAtStart() As Boolean
        Get
            Return m_WordWasOpen
        End Get
    End Property
    ''' <summary>
    ''' Information if a Word application could be found.
    ''' </summary>
    ReadOnly Property WordExists() As Boolean
        Get
            If m_WdApp IsNot Nothing Then
                Return True
            End If
            Return False
        End Get
    End Property
    ''' <summary>
    ''' Gets or sets the visbility of the Word application.
    ''' </summary>
    Property Visible() As Boolean
        Get
            If m_WdApp IsNot Nothing Then
                Return m_WdApp.Visible
            End If
        End Get
        Set(ByVal value As Boolean)
            If m_WdApp IsNot Nothing Then
                m_WdApp.Visible = value
            End If
        End Set
    End Property
#End Region

#Region " Methods "
    ''' <summary>
    ''' Returns the version number of the Word application.
    ''' </summary>
    Function VersionNumber() As Integer
        If m_WdApp Is Nothing Then
            Return 0
        Else
            Dim txt As String = m_WdApp.Version
            Dim Idx As Integer = txt.IndexOf(".")
            If Idx > 0 Then
                txt = txt.Substring(0, Idx)
            End If

            Dim RetValue As Integer = 0
            Try
                RetValue = CInt(txt)
            Catch ex As Exception

            End Try
            Return RetValue
        End If
    End Function
    ''' <summary>
    ''' Activates the Word application.
    ''' </summary>
    Public Sub Activate()
        If m_WdApp IsNot Nothing Then
            m_WdApp.Activate()
        End If
    End Sub
    ''' <summary>
    ''' Opens or creates a new Word document and returns it.
    ''' </summary>
    ''' <param name="Path">Path of a document to open.</param>
    ''' <param name="TemplatePath">Template path for a new created document.</param>
    ''' <param name="NewTemplate">True, if a template is to create.</param>
    ''' <param name="Visible">True, if show document creation visible.</param>
    Public Function OpenDocument(ByVal Path As String, Optional ByVal TemplatePath As String = "", Optional ByVal NewTemplate As Boolean = False, Optional ByVal Visible As Boolean = False) As WordDocument
        If m_WdApp Is Nothing Then Return Nothing

        Dim SD As WordDocument = Nothing

        Dim Docs As Object = m_WdApp.Documents
        Dim Dok As Object

        If IO.File.Exists(Path) Then
            ' Path exists
            ' First search for open documents
            If Docs.Count > 0 Then
                For I As Integer = 1 To Docs.Count
                    Dok = Docs.Item(I)
                    If Dok.FullName = Path Then
                        SD = New WordDocument(Dok, True)
                        Exit For
                    End If
                    ReleaseObject(Dok)
                Next
            End If

            ' if document is close then open it
            If SD Is Nothing Then SD = New WordDocument(Docs.Open(Path), False)

        Else
            ' Path not exists
            If TemplatePath = "" Then
                ' create new document without template
                SD = New WordDocument(Docs.Add(, NewTemplate, 0, Visible), False)
            Else
                ' create new document with template
                SD = New WordDocument(Docs.Add(TemplatePath, NewTemplate, 0, Visible), False)
            End If
        End If

        ReleaseObject(Docs)

        Return SD
    End Function
    ''' <summary>
    ''' Closes the Word application.
    ''' </summary>
    Public Sub Close()
        If m_WdApp IsNot Nothing Then
            If Not m_WordWasOpen Then
                m_WdApp.Quit(0)
            End If
        End If
        Dispose()
    End Sub

#Region " IDisposable Support "
    ''' <summary>
    ''' IDisposible support.
    ''' </summary>
    ''' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.m_disposedValue Then
            If disposing Then
            End If
            ReleaseObject(m_WdApp)

        End If
        Me.m_disposedValue = True
    End Sub

    ''' <summary>
    ''' IDisposible support.
    ''' </summary>
    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

    ''' <summary>
    ''' Releases marshalled objects.
    ''' </summary>
    <System.Diagnostics.DebuggerStepThrough()> _
    Private Sub ReleaseObject(ByVal obj As Object)
        If obj IsNot Nothing Then
            Try
                System.Runtime.InteropServices.Marshal.ReleaseComObject(obj)
            Catch
            Finally
                obj = Nothing
            End Try
        End If
    End Sub
#End Region

End Class






