
''' <summary>
''' Class to hold a Word document reference and to dispose it after use
''' </summary>
Friend Class WordDocument
    Implements IDisposable

#Region " Fields "
    ''' <summary>
    ''' Reference to the Word document.
    ''' </summary>
    Private m_Document As Object
    ''' <summary>
    ''' True, if the Word document was open at start.
    ''' </summary>
    Private m_WasOpen As Boolean
    ''' <summary>
    ''' Support for IDisposible.
    ''' </summary>
    Private m_disposedValue As Boolean = False
#End Region

#Region " Constructor "
    ''' <summary>
    ''' Default Constructor.
    ''' </summary>
    ''' <param name="Doc">Referenced Word document object.</param>
    ''' <param name="WasOpen">Must be True, if the document was open at start.</param>
    Public Sub New(ByVal Doc As Object, ByVal WasOpen As Boolean)
        m_Document = Doc
        m_WasOpen = WasOpen
    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Gets or sets the visibility of the document in Word.
    ''' </summary>
    Property Visible() As Boolean
        Get
            Dim Wndw As Object = m_Document.ActiveWindow
            Dim Result As Boolean = Wndw.Visible
            ReleaseObject(Wndw)
            Return Result
        End Get
        Set(ByVal value As Boolean)
            Dim Wndw As Object = m_Document.ActiveWindow
            Wndw.Visible = value
            ReleaseObject(Wndw)
        End Set
    End Property
#End Region

#Region " Methods "
    ''' <summary>
    ''' Activates the Word document.
    ''' </summary>
    Public Sub Activate()
        m_Document.Activate()
        m_Document.ActiveWindow.View.Type = Document.WdViewType.wdPrintView
    End Sub

    ''' <summary>
    ''' Writes content from a range description to a Word document
    ''' </summary>
    ''' <param name="Rng">Range description.</param>
    ''' <param name="FirstPageTopMargin">Value to set the top margin on the first page of the document.</param>
    Public Sub Write(ByVal Rng As Document.Range, Optional ByVal FirstPageTopMargin As Single = 0.0F)

        If Rng.Paragraphs.Count = 0 AndAlso Rng.Tables.Count = 0 Then
            Dim Rng1 As Object = m_Document.Range

            For Each BM As Document.Bookmark In Rng.Bookmarks
                BM.Fill(Rng1)
            Next

            ReleaseObject(Rng1)
        Else
            Dim Rng1 As Object = m_Document.Range
            Rng.Write(Rng1)
            ReleaseObject(Rng1)

            If FirstPageTopMargin <> 0.0F Then
                Dim Section As Object = m_Document.Sections(1)
                Dim PageSetup As Object = Section.PageSetup
                PageSetup.TopMargin = FirstPageTopMargin
                ReleaseObject(PageSetup)
                ReleaseObject(Section)
            End If

        End If
    End Sub

    ''' <summary>
    ''' Writes a string into a hidden shape into the Word document.
    ''' </summary>
    ''' <param name="Text">String to write.</param>
    Public Sub WriteHiddenText(ByVal Text As String)
        Dim myTBox As Object = Nothing

        Dim Sections As Object = m_Document.Sections
        Dim Sec As Object = Sections.Item(1)
        Dim Headers As Object = Sec.Headers
        Dim Itm As Object = Headers.Item(2)
        Dim Shapes As Object = Itm.Shapes
        Dim Shp As Object = Nothing

        If Shapes.Count > 0 Then
            For Each Shp In Shapes
                If Shp.Name = "DocProps" Then
                    myTBox = Shp
                    Exit For
                End If
                ReleaseObject(Shp)
            Next
        End If

        If myTBox Is Nothing Then
            myTBox = Shapes.AddTextbox(1, 0, 0, 15, 15)
            myTBox.Name = "DocProps"
        End If

        myTBox.Visible = 0

        Dim TxtFrame As Object = myTBox.TextFrame
        Dim TextRange As Object = TxtFrame.TextRange
        TextRange.Text = Text

        ReleaseObject(TextRange)
        ReleaseObject(TxtFrame)

        ReleaseObject(Shp)

        ReleaseObject(Shapes)
        ReleaseObject(Itm)
        ReleaseObject(Headers)
        ReleaseObject(Sec)
        ReleaseObject(Sections)
        ReleaseObject(myTBox)

    End Sub
    ''' <summary>
    ''' Reads text from a special hidden shape in the document.
    ''' </summary>
    Public Function ReadHiddenText() As String
        Dim Result As String

        Dim myTBox As Object = Nothing
        Dim Sections As Object = m_Document.Sections
        Dim Sec As Object = Sections.Item(1)
        Dim Headers As Object = Sec.Headers
        Dim Itm As Object = Headers.Item(2)
        Dim Shapes As Object = Itm.Shapes
        Dim Shp As Object = Nothing

        If Shapes.Count > 0 Then
            For Each Shp In Shapes
                If Shp.Name = "DocProps" Then
                    myTBox = Shp
                    Exit For
                End If
                ReleaseObject(Shp)
            Next
        End If

        If myTBox IsNot Nothing Then
            Dim TxtFrame As Object = myTBox.TextFrame
            Dim TextRange As Object = TxtFrame.TextRange

            Result = TextRange.Text

            ReleaseObject(TextRange)
            ReleaseObject(TxtFrame)
        Else
            Result = ""
        End If


        ReleaseObject(Shp)

        ReleaseObject(Shapes)
        ReleaseObject(Itm)
        ReleaseObject(Headers)
        ReleaseObject(Sec)
        ReleaseObject(Sections)
        ReleaseObject(myTBox)

        Return Result
    End Function
    ''' <summary>
    ''' Saves changes in the Word document.
    ''' </summary>
    Public Sub Save()
        m_Document.Save()
    End Sub
    ''' <summary>
    ''' Saves the Word document to a specified path.
    ''' </summary>
    ''' <param name="Path">Document path.</param>
    Public Sub SaveAs(ByVal Path As String)
        m_Document.SaveAs(Path)
    End Sub

    ''' <summary>
    ''' Closes the Word document.
    ''' </summary>
    Public Sub Close()
        If Not m_WasOpen Then m_Document.Close(0)
        Dispose()
    End Sub
#Region " IDisposable Support "
    ''' <summary>
    ''' Idisposible support
    ''' </summary>
    ''' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.m_disposedValue Then
            If disposing Then

            End If
            ReleaseObject(m_Document)

        End If
        Me.m_disposedValue = True
    End Sub

    ''' <summary>
    ''' Idisposible support
    ''' </summary>
    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

    ''' <summary>
    ''' Releases marshalled objects.
    ''' </summary>
    <System.Diagnostics.DebuggerStepThrough()> _
    Private Sub ReleaseObject(ByVal obj As Object)
        If obj IsNot Nothing Then
            Try
                System.Runtime.InteropServices.Marshal.ReleaseComObject(obj)
            Catch
            Finally
                obj = Nothing
            End Try
        End If
    End Sub
#End Region

End Class


